n <- read.table("http://ashipunov.info/data/nanodrop.txt", h=T, sep="\t", as.is=T)
nn <- n[!(n$CONC > 50 | n$PURITY > 5),]
AGE <- max(nn$YEAR, na.rm=T) - nn$YEAR
summary(nn)

# ===
# CONC ~ AGE

# with(nn, scatter.smooth(AGE, CONC))
dna.lm <- lm(CONC ~ AGE, data=nn)
sink("res/nanodrop_res.txt", split=T)
summary(dna.lm)
sink()

# plot(dna.lm)
new.frame <- data.frame(AGE=seq(min(AGE, na.rm=T), max(AGE, na.rm=T), along.with=AGE))
predicted.points <- predict(dna.lm, interval="confidence", newdata=new.frame)

pdf("res/conc_age.pdf")
plot(CONC ~ AGE, data=nn, xlab="Age of Sample", ylab="Concentration of DNA", main="")
abline(dna.lm)
matlines(new.frame$AGE, predicted.points, col="black", lty=c(0,2,2))
dev.off()

# ===
# PURITY ~ AGE

# with(nn, scatter.smooth(AGE, PURITY))
dna2.lm <- lm(PURITY ~ AGE, data=nn)
sink("res/nanodrop_res.txt", split=T, append=T)
summary(dna2.lm)
sink()

# plot(dna2.lm)
predicted.points <- predict(dna2.lm, int="c", newdata=new.frame)

pdf("res/purity_age.pdf")
plot(PURITY ~ AGE, data=nn, xlab="Age of Sample", ylab="Purity of DNA", main="")
abline(dna2.lm)
matlines(new.frame$AGE, predicted.points, col="black", lty=c(0,2,2))
dev.off()

# ===
# CONC ~ WEIGHT

# with(nn, scatter.smooth(WEIGHT, CONC))
dna3.lm <- lm(CONC ~ WEIGHT, data=nn)
sink("res/nanodrop_res.txt", split=T, append=T)
summary(dna3.lm)
sink()

new.frame2 <- data.frame(WEIGHT=seq(min(nn$WEIGHT, na.rm=T), max(nn$WEIGHT, na.rm=T), along.with=nn$WEIGHT))
predicted.points2 <- predict(dna3.lm, interval="confidence", newdata=new.frame2)

pdf("res/conc_weight.pdf")
plot(CONC ~ WEIGHT, data=nn, xlab="Concentration of DNA", ylab="Weight of Sample", main="")
abline(dna3.lm)
matlines(new.frame2$WEIGHT, predicted.points2, col="black", lty=c(0,2,2))
dev.off()

# with(nn, scatter.smooth(WEIGHT, PURITY)) # horizontal line
# with(nn, scatter.smooth(CONC, PURITY)) # has no sense

# ===
# Two protocols

sink("res/nanodrop_res.txt", split=T, append=T)
wilcox.test(CONC ~ PROTOCOL, data=nn)
wilcox.test(PURITY ~ PROTOCOL, data=nn)
sink()

pdf("res/two_protocols.pdf")
oldpar <- par(mfrow=c(1,2))
boxplot(CONC ~ PROTOCOL, data=nn, main="DNA Concentration")
boxplot(PURITY ~ PROTOCOL, data=nn, main="DNA Purity")
par(oldpar)
dev.off()

# ===
# Success of PCR (log model)

# !!!

